<?php

namespace Drupal\Tests\lingotek\Functional;

use Drupal\Core\Url;
use Drupal\lingotek\Lingotek;
use Drupal\Tests\BrowserTestBase;
use GuzzleHttp\Cookie\CookieJar;

/**
 * Base class for Lingotek test. Performs authorization of the account.
 */
abstract class LingotekTestBase extends BrowserTestBase {

  /**
   * The cookie jar holding the testing session cookies for Guzzle requests.
   *
   * @var \GuzzleHttp\Client;
   */
  protected $client;

  /**
   * The Guzzle HTTP client.
   *
   * @var \GuzzleHttp\Cookie\CookieJar;
   */
  protected $cookies;

  /*
   * Modules to install.
   *
   * @var array
   */
  public static $modules = ['lingotek', 'lingotek_test'];

  /**
   * @var \Drupal\Core\Session\AccountInterface
   *   Minimal Lingotek translation manager user.
   */
  protected $translationManagerUser;

  protected function setUp() {
    parent::setUp();

    // Login as admin.
    $this->drupalLogin($this->rootUser);
    $this->connectToLingotek();

    $this->client = $this->getHttpClient();

    $this->createTranslationManagerUser();
  }

  /**
   * Creates a translation manager role and a user with the minimal
   * Lingotek translation management permissions.
   */
  protected function createTranslationManagerUser() {
    $this->translationManagerUser = $this->drupalCreateUser([
      'assign lingotek translation profiles',
      'manage lingotek translations',
      'access administration pages',
    ]);
  }

  /**
   * Create a new image field.
   *
   * @param string $name
   *   The name of the new field (all lowercase).
   * @param string $type_name
   *   The bundle that this field will be added to.
   * @param string $entity_type_id
   *   The entity type that this field will be added to. Defaults to 'node'
   * @param array $storage_settings
   *   A list of field storage settings that will be added to the defaults.
   * @param array $field_settings
   *   A list of instance settings that will be added to the instance defaults.
   * @param array $widget_settings
   *   A list of widget settings that will be added to the widget defaults.
   */
  public function createImageField($name, $type_name, $entity_type_id = 'node', $storage_settings = [], $field_settings = [], $widget_settings = []) {
    entity_create('field_storage_config', [
      'field_name' => $name,
      'entity_type' => $entity_type_id,
      'type' => 'image',
      'settings' => $storage_settings,
      'cardinality' => !empty($storage_settings['cardinality']) ? $storage_settings['cardinality'] : 1,
    ])->save();

    $field_config = entity_create('field_config', [
      'field_name' => $name,
      'label' => $name,
      'entity_type' => $entity_type_id,
      'bundle' => $type_name,
      'required' => !empty($field_settings['required']),
      'settings' => $field_settings,
    ]);
    $field_config->save();

    entity_get_form_display($entity_type_id, $type_name, 'default')
      ->setComponent($name, [
        'type' => 'image_image',
        'settings' => $widget_settings,
      ])
      ->save();

    entity_get_display($entity_type_id, $type_name, 'default')
      ->setComponent($name)
      ->save();

    return $field_config;

  }

  protected function connectToLingotek() {
    $this->drupalGet('admin/lingotek/setup/account');
    $this->clickLink('Connect Lingotek Account');
    $this->drupalPostForm(NULL, ['community' => 'test_community'], 'Next');
    $this->drupalPostForm(NULL, [
      'project' => 'test_project',
      'vault' => 'test_vault'
    ], 'Save configuration');
  }

  /**
   * Go to the content bulk management form.
   *
   * @param string $entity_type_id
   *   Entity type ID we want to manage in bulk. By default is node.
   */
  protected function goToContentBulkManagementForm($entity_type_id = 'node') {
    $this->drupalGet('admin/lingotek/manage/' . $entity_type_id);
  }


  /**
   * Go to the config bulk management form and filter one kind of configuration.
   *
   * @param string $filter
   *   Config name of the filter to apply. By default is NULL and will use the
   *   current one.
   */
  protected function goToConfigBulkManagementForm($filter = NULL) {
    $this->drupalGet('admin/lingotek/config/manage');

    if ($filter !== NULL) {
      $edit = ['filters[wrapper][bundle]' => $filter];
      $this->drupalPostForm(NULL, $edit, t('Filter'));
    }
  }

  /**
   * Asserts if the uploaded data contains the expected number of fields.
   *
   * @param array $data
   *   The uploaded data.
   * @param $count
   *   The expected number of items.
   *
   * @return bool
   *   TRUE if the assertion succeeded, FALSE otherwise.
   */
  protected function assertUploadedDataFieldCount(array $data, $count) {
    // We have to add one item because of the metadata we include.
    return $this->assertEqual($count + 1, count($data));
  }

  /**
   * Asserts if there are a number of documents with a given status and language
   * label as source.
   *
   * @param string $status
   *   The status we are looking for. Use Lingotek constants.
   * @param string $languageLabel
   *   The language label of the source.
   * @param int $count
   *   The expected number of items.
   * @param string $message
   *   (optional) A message to display with the assertion. Do not translate
   *   messages: use \Drupal\Component\Utility\SafeMarkup::format() to embed
   *   variables in the message text, not t(). If left blank, a default message
   *   will be displayed.
   *
   * @return bool
   *   TRUE if the assertion succeeded, FALSE otherwise.
   */
  protected function assertSourceStatusStateCount($status, $languageLabel, $count, $message = '') {
    $statusCssClass = 'source-' . strtolower($status);
    if ($status === Lingotek::STATUS_CURRENT) {
      // There is no link or anchor when the status is current.
      $statusCount = $this->xpath("//span[contains(@class,'language-icon') and contains(@class, '$statusCssClass') and contains(text(), '$languageLabel')]");
    }
    else {
      $statusCount = $this->xpath("//span[contains(@class,'language-icon') and contains(@class, '$statusCssClass')]/a[contains(text(), '$languageLabel')]");
    }
    return $this->assertEqual(count($statusCount), $count, $message);
  }

  /**
   * Asserts a given index of the management table shows a given profile.
   *
   * @param int $index
   *   The index of the table to check.
   * @param string|null $profile
   *   The profile to verify.
   */
  protected function assertManagementFormProfile($index, $profile) {
    $elements = $this->xpath("//*[@id='edit-table']/tbody/tr[$index]/td[6]");
    if ($profile === NULL) {
      $this->assertEqual(0, count($elements), "Profile for $index is shown as empty");
    }
    else {
      $shown_profile = $elements[0]->getHtml();
      $this->assertEqual($profile, $shown_profile, "Profile for $index is shown as $profile");
    }
  }

  /**
   * Create and publish a node.
   *
   * @param array $edit
   *   Field data in an associative array.
   * @param string $bundle
   *   The bundle of the node to be created.
   */
  protected function saveAndPublishNodeForm(array $edit, $bundle = 'article') {
    $path = ($bundle !== NULL) ? "node/add/$bundle" : NULL;
    if (floatval(\Drupal::VERSION) >= 8.4) {
      if (\Drupal::moduleHandler()->moduleExists('content_moderation')) {
        $edit['moderation_state[0][state]'] = 'published';
        $this->drupalPostForm($path, $edit, t('Save'));
      }
      else {
        $edit['status[value]'] = TRUE;
        $this->drupalPostForm($path, $edit, t('Save'));
      }
    }
    else {
      if (\Drupal::moduleHandler()->moduleExists('content_moderation')) {
        $this->drupalPostForm($path, $edit, t('Save and Publish'));
      }
      else {
        $this->drupalPostForm($path, $edit, t('Save and publish'));
      }
    }
  }

  protected function saveAndUnpublishNodeForm(array $edit, $nid) {
    $path = ($nid !== NULL) ? "node/$nid/edit" : NULL;
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['status[value]'] = FALSE;
      $this->drupalPostForm($path, $edit, t('Save'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save and unpublish'));
    }
  }

  protected function saveAsUnpublishedNodeForm(array $edit, $bundle = 'article') {
    $path = ($bundle !== NULL) ? "node/add/$bundle" : NULL;
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['status[value]'] = FALSE;
      $this->drupalPostForm($path, $edit, t('Save'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save as unpublished'));
    }
  }

  protected function saveAsRequestReviewNodeForm(array $edit, $bundle = 'article') {
    $path = ($bundle !== NULL) ? "node/add/$bundle" : NULL;
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['moderation_state[0][state]'] = 'needs_review';
      $this->drupalPostForm($path, $edit, t('Save'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save and Request Review'));
    }
  }

  protected function editAsRequestReviewNodeForm($path, array $edit) {
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['moderation_state[0][state]'] = 'needs_review';
      $this->drupalPostForm($path, $edit, t('Save'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save and Request Review (this translation)'));
    }
  }

  protected function saveAsNewDraftNodeForm(array $edit, $bundle = 'article') {
    $path = ($bundle !== NULL) ? "node/add/$bundle" : NULL;
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['moderation_state[0][state]'] = 'draft';
      $this->drupalPostForm($path, $edit, t('Save'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save and Create New Draft'));
    }
  }

  protected function editAsNewDraftNodeForm($path, array $edit) {
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['moderation_state[0][state]'] = 'draft';
      $this->drupalPostForm($path, $edit, t('Save'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save and Create New Draft (this translation)'));
    }
  }

  protected function saveAndKeepPublishedNodeForm(array $edit, $nid) {
    $path = ($nid !== NULL) ? "node/$nid/edit" : NULL;
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['status[value]'] = TRUE;
      $this->drupalPostForm($path, $edit, t('Save'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save and keep published'));
    }
  }

  protected function saveAndKeepPublishedThisTranslationNodeForm(array $edit, $nid) {
    $path = ($nid !== NULL) ? "node/$nid/edit" : NULL;
    if (floatval(\Drupal::VERSION) >= 8.4) {
      $edit['status[value]'] = TRUE;
      $this->drupalPostForm($path, $edit, t('Save (this translation)'));
    }
    else {
      $this->drupalPostForm($path, $edit, t('Save and keep published (this translation)'));
    }
  }

  /**
   * Configure content moderation.
   *
   * @param string $workflow_id
   *   The workflow id to be configured.
   * @param array $entities_map
   *   The entities and bundles map that wants to be enabled for a given workflow.
   *   Array in the form: [entity_type => [bundle1, bundle2]].
   */
  protected function configureContentModeration($workflow_id, $entities_map) {
    if (floatval(\Drupal::VERSION) >= 8.4) {
      foreach ($entities_map as $entity_type_id => $bundles) {
        $edit = [];
        foreach ($bundles as $bundle) {
          $edit["bundles[$bundle]"] = $bundle;
        }
        $this->drupalPostForm("admin/config/workflow/workflows/manage/$workflow_id/type/$entity_type_id", $edit, 'Save');
      }
    }
    else {
      if (isset($entities_map['node'])) {
        foreach ($entities_map['node'] as $bundle) {
          $this->drupalPostForm("/admin/structure/types/manage/$bundle/moderation", ['workflow' => $workflow_id], t('Save'));
        }
      }
    }
  }

  /**
   * Assert that there is a link to the workbench in a new tab.
   *
   * @param string $document_id
   *   The document id.
   * @param $langcode
   *   The language code.
   * @param $locale
   *   The Lingotek locale.
   */
  protected function assertLinkToWorkbenchInNewTabInSinglePage($document_id, $langcode, $locale) {
    $this->assertLinkByHref("/admin/lingotek/workbench/$document_id/$langcode");
    $url = Url::fromRoute('lingotek.workbench', [
      'doc_id' => $document_id,
      'locale' => $locale
    ])->toString();

    $this->assertRaw('<a href="' . $url . '" target="_blank">', 'The workbench link opens in a new tab.');
  }

  /**
   * Assert that a content target has the given status.
   *
   * @param string $language
   *   The target language.
   * @param string $status
   *   The status.
   */
  protected function assertTargetStatus($language, $status) {
    $status_target = $this->xpath("//a[contains(@class,'language-icon') and contains(@class,'target-" . $status . "')  and contains(text(), '" . $language . "')]");
    $this->assertEqual(count($status_target), 1, 'The target ' . $language . ' has been marked with status ' . $status . '.');
  }

  /**
   * Assert that a content target has not been marked as error.
   *
   * @param string $label
   *   The label of the row.
   * @param string $language
   *   The target language.
   * @param string $locale
   *   The target locale.
   */
  protected function assertNoTargetError($label, $language, $locale) {
    $source_error = $this->xpath("//span[contains(@class,'language-icon') and contains(@class,'target-error')  and ./a[contains(text(), '" . $language . "')]]");
    $this->assertEqual(count($source_error), 0, 'The target ' . $language . ' has not been marked as error.');
    $this->assertNoText($label . ' ' . $locale . ' translation download failed. Please try again.');
  }

  /**
   * Assert that a config target has not been marked as error.
   *
   * @param string $label
   *   The label of the row.
   * @param string $language
   *   The target language.
   * @param string $locale
   *   The target locale.
   */
  protected function assertNoConfigTargetError($label, $language, $locale) {
    $source_error = $this->xpath("//span[contains(@class,'language-icon') and contains(@class,'target-error')  and ./a[contains(text(), '" . $language . "')]]");
    $this->assertEqual(count($source_error), 0, 'The target ' . $language . ' has not been marked as error.');
    $this->assertNoText($label . ' ' . $locale . ' translation download failed. Please try again.');
  }

  /**
   * Obtain the HTTP client and set the cookies.
   *
   * @return \GuzzleHttp\Client
   *   The client with BrowserTestBase configuration.
   */
  protected function getHttpClient() {
    // Similar code is also employed to test CSRF tokens.
    // @see \Drupal\Tests\system\Functional\CsrfRequestHeaderTest::testRouteAccess()
    $domain = parse_url($this->getUrl(), PHP_URL_HOST);
    $session_id = $this->getSession()->getCookie($this->getSessionName());
    $this->cookies = CookieJar::fromArray([$this->getSessionName() => $session_id], $domain);
    return $this->getSession()->getDriver()->getClient()->getClient();
  }

}
